
//*******************************************
//* TP3: Liste d'entier simplement chainee *
//*******************************************

#include <iostream>

using namespace std;

//*****************************************
//* Structure de liste simplement chainee *
//*****************************************

struct Element{
  int valeur;
  Element* suivant; 
};

struct Liste{
  Element* tete;
};


//**************
//* Exercice 1 *
//**************

// Ecrire une fonction affiche_liste(l) affichant la liste chainee l
// Exemple :
//  affiche_liste(l1) -> ()
//  affiche_liste(l2) -> (5)
//  affiche_liste(l3) -> (2,5,8,3,2,4)

void affiche_liste(Liste& l){
  Element* courant=l.tete;
  if(courant==NULL){
    //La liste est vide
    cout<<"()"<<endl;
    return;
  }
  else{
    //On affiche le premier element
    cout<<'('<<courant->valeur;
    //On se deplace sur le deuxième
    courant=courant->suivant;
    
    //Boucle principale
    while(courant!=NULL){
      //Affichage courant->valeur
      cout<<','<<courant->valeur;
      //Deplacement du curseur courant
      courant=courant->suivant;
    }
    cout<<')'<<endl;
  }
}
//**************
//* Exercice 2 *
//**************

// Ecrire les fonctions suivantes.

// 1. est_vide(l) qui teste si la liste l est vide.
bool est_vide(Liste& l){
  return l.tete==NULL;
}

// 2. taille(l) qui retourne le nombre d'elements de la liste l.
int taille(Liste l){
  Element* courant=l.tete;
  int n=0;
  while(courant!=NULL){
    n=n+1;
    courant=courant->suivant;
  }
  return n;
}

// 3. inserer_debut(l,v) qui insere v au debut de la liste l.
void inserer_debut(Liste& l,int v){
  Element* nouveau=new Element;
  nouveau->valeur=v;
  nouveau->suivant=l.tete;
  l.tete=nouveau;
}

// 4. nombre(l,v) qui retourne le nombre d'occurences de la valeur v
// dans la liste l.

int nombre(Liste& l,int v){
  int n=0;
  Element* courant=l.tete;
  while(courant!=NULL){
    if(courant->valeur==v){
      n=n+1;
    }
    courant=courant->suivant;
  }
  return n;
}

// 5. remplace(l,v,w) qui remplace la valeur v par la valeur w dans la liste l.

int remplace(Liste& l,int v,int w){
  Element* courant=l.tete;
  while(courant!=NULL){
    if(courant->valeur==v){
      courant->valeur=w;
    }
    courant=courant->suivant;
  }
}

//**************
//* Exercice 3 *
//**************

// Ecrire les fonctions suivantes

// 1. inserer_fin(l,v) qui insere la valeur v a la fin de la liste l.

void inserer_fin(Liste& l,int v){
  Element* nouveau=new Element;
  nouveau->valeur=v;
  nouveau->suivant=NULL;
  Element* courant=l.tete;
  if(courant!=NULL){
    while(courant->suivant!=NULL){
      courant=courant->suivant;
    }
    courant->suivant=nouveau;
  }
  else{
    //La liste est vide
    l.tete=nouveau;
  }			      
}

// 2. recherche(l,v) qui retourne true si l'element v est present dans
//    la liste l et false sinon.
bool recherche(Liste& l,int v){
  Element* courant=l.tete;
  while(courant!=NULL){
    if(courant->valeur==v){
      return true;
    }
    courant=courant->suivant;
  }
  return false;
}

// 3. supprime(l,n) qui supprime l'element numero n de la liste l.
//    Le premier element porte le numéro 0.

void supprime(Liste& l,int n){
  if(n>0){
    Element* courant=l.tete;
    int cpt=n;
    while(courant!=NULL and cpt>1){
      courant=courant->suivant;
      --cpt;
    }
    if(courant!=NULL){
      //On a donc nécessairement cpt=1
      Element* temp=courant->suivant;
      //temp pointe sur le noeud à supprimer
      if(temp!=NULL){
	//Le noeud a supprimer existe
	courant->suivant=temp->suivant;
	delete temp;
      }
      else{
	cout<<"Attention liste trop courte d'un élément."<<endl;
      }
    }
    else{
      cout<<"Attention liste trop courte."<<endl;
    }
  }
  else{
    //n=0
    if(l.tete==NULL){
      cout<<"Attention liste vide"<<endl;
    }
    else{
      Element* temp=l.tete;
      l.tete=temp->suivant;
      delete temp;
    } 
  }
}

// 4. supprime_valeur(l,v) qui supprime la valeur v de la liste l.
//    La fonction retournera le nombre de valeur supprimees.

int supprime_valeur(Liste& l,int v){
  int cpt=0;
  //Suppression des v en tête de l
  while(l.tete!=NULL and l.tete->valeur==v){
    ++cpt;
    Element* temp=l.tete;
    l.tete=temp->suivant;
    delete temp;
  }
  Element* courant=l.tete;
  if(courant==NULL) return cpt; //La liste est maintenant vide.
  //Sinon la liste est non vide
  while(courant->suivant!=NULL){
    if(courant->suivant->valeur==v){
      ++cpt;
      Element* temp=courant->suivant;
      courant->suivant=courant->suivant->suivant;
      delete temp;
    }
    else{
      courant=courant->suivant;
    }
  }
  return cpt;
}

// 5. insere_apres(l,v,w) qui insere un element de valeur w apres la
//    premiere occurence de la valeur v dans la liste l. Ne fait rien si
//    v n'est pas present dans l.

void insere_apres(Liste& l,int v,int w){
  Element* courant=l.tete;
  while(courant!=NULL){
    if(courant->valeur==v){
      Element* nouveau=new Element;
      nouveau->valeur=w;
      nouveau->suivant=courant->suivant;
      courant->suivant=nouveau;
      return;
    }
    courant=courant->suivant;
  }
}

// 6. insere_avant(l,v,w) qui insere un noeud portant la valeur w avant la
//    premiere occurence de la valeur v dans la liste l. Ne fait rien si
//    v n'est pas present dans l.

void insere_avant(Liste& l,int v,int w){
  if(l.tete!=NULL){
    //Liste l non vide
    if(l.tete->valeur==v){
      Element* nouveau=new Element;
      nouveau->valeur=w;
      nouveau->suivant=l.tete;
      l.tete=nouveau;
    }
    else{
      Element* courant=l.tete;
      while(courant->suivant!=NULL){
	if(courant->suivant->valeur==v){
	  Element* nouveau=new Element;
	  nouveau->valeur=w;
	  nouveau->suivant=courant->suivant;
	  courant->suivant=nouveau;
	  return;
	}
	courant=courant->suivant;
      }
    }
  }
}

//**************
//* Exercice 4 *
//**************

// Ecrire une fonction inverse(l) qui retourne une liste obtenue en inversant
// la liste l

Liste inverse(Liste& l){
  Element *courant=l.tete;
  Element *courant_inv=NULL;
  while(courant!=NULL){
    Element* nouveau=new Element;
    nouveau->valeur=courant->valeur;
    nouveau->suivant=courant_inv;
    courant_inv=nouveau;
    courant=courant->suivant;
  }
  Liste linv;
  linv.tete=courant_inv;
  return linv;
}

//**************
//* Exercice 5 *
//**************

// Refaire l'exercice de simulation de train du TD 3.


//*****************
//* Fonction Main *
//*****************

int main(){
  //**********************************************************
  //* Voici quelques listes utiles pour tester vos fonctions *
  //**********************************************************

  // Liste vide ()
  Liste l1; //()
  l1.tete=NULL;

  // Liste (5)
  Liste l2;
  Element* e21=new Element;
  l2.tete=e21;
  e21->valeur=5;
  e21->suivant=NULL;

  //Liste (2,5,8,3,2,4)
  Liste l3;
  Element* e31=new Element;
  Element* e32=new Element;
  Element* e33=new Element;
  Element* e34=new Element;
  Element* e35=new Element;
  Element* e36=new Element;
  l3.tete=e31;
  e31->suivant=e32;
  e32->suivant=e33;
  e33->suivant=e34;
  e34->suivant=e35;
  e35->suivant=e36;
  e36->suivant=NULL;
  e31->valeur=2;
  e32->valeur=5;
  e33->valeur=8;
  e34->valeur=3;
  e35->valeur=2;
  e36->valeur=4;

  //****************************
  //* Tester vos fonctions ici *
  //****************************

  cout<<"**************"<<endl;
  cout<<"* Exercice 1 *"<<endl;
  cout<<"**************"<<endl;
  cout<<endl;
  affiche_liste(l1);
  affiche_liste(l2);
  affiche_liste(l3);
  cout<<endl;
  cout<<"**************"<<endl;
  cout<<"* Exercice 2 *"<<endl;
  cout<<"**************"<<endl;
  cout<<endl;
  cout<<"Question 1"<<endl;
  cout<<"l1 vide ? "<<est_vide(l1)<<endl;
  cout<<"l2 vide ? "<<est_vide(l2)<<endl;
  cout<<"l3 vide ? "<<est_vide(l3)<<endl;
  cout<<endl;
  cout<<"Question 2"<<endl;
  cout<<"taille l1 : "<<taille(l1)<<endl;
  cout<<"taille l2 : "<<taille(l2)<<endl;
  cout<<"taille l3 : "<<taille(l3)<<endl;
  cout<<endl;
  cout<<"Question 3"<<endl;
  cout<<"Ajoute -5 au début de l3 : ";
  inserer_debut(l3,-5);
  affiche_liste(l3);
  cout<<endl;
  cout<<"Question 4"<<endl;
  cout<<"Nombre de 2 dans l3 : "<<nombre(l3,2)<<endl;
  cout<<"Nombre de 5 dans l3 : "<<nombre(l3,5)<<endl;
  cout<<"Nombre de -1 dans l3 : "<<nombre(l3,-1)<<endl;
  cout<<endl;
  cout<<"Question 5"<<endl;
  cout<<"Remplace -5 par 1 dans l3 : ";
  remplace(l3,-5,1);
  affiche_liste(l3);
  cout<<"Remplace 0 par 7 dans l3 : ";
  remplace(l3,0,7);
  affiche_liste(l3);
  cout<<endl;
  cout<<"**************"<<endl;
  cout<<"* Exercice 3 *"<<endl;
  cout<<"**************"<<endl;
  cout<<endl;
  cout<<"Question 1"<<endl;
  cout<<"Ajout de 6 à la fin de l3 : ";
  inserer_fin(l3,6);
  affiche_liste(l3);
   cout<<endl;
  cout<<"Question 2"<<endl;
  cout<<"Recherche 5 dans l3 : "<<recherche(l3,5)<<endl;
  cout<<"Recherche 0 dans l3 : "<<recherche(l3,0)<<endl;
  cout<<endl;
  cout<<"Question 3"<<endl;
  cout<<"Supprime élément d'indice 4 dans l3 : ";
  supprime(l3,4);
  affiche_liste(l3);
  cout<<"Supprime élément d'indice 0 dans l3 : ";
  supprime(l3,0);
  affiche_liste(l3);
  cout<<"Supprime élément d'indice 6 dans l3 : ";
  supprime(l3,6);
  affiche_liste(l3);
  cout<<"Supprime élément d'indice 7 dans l3 : ";
  supprime(l3,7);
  affiche_liste(l3);
  cout<<"Supprime élément d'indice 0 dans l1 : ";
  supprime(l1,0);
  affiche_liste(l1);
  cout<<endl;
  cout<<"Question 4"<<endl;
  inserer_debut(l3,2);
  inserer_debut(l3,2);
  inserer_debut(l3,0);
  inserer_debut(l3,1);
  inserer_debut(l3,2);
  inserer_debut(l3,2);
  inserer_debut(l3,2);
  cout<<"l3 = ";
  affiche_liste(l3);
  cout<<"Suppression du 2 dans l3 : "<<supprime_valeur(l3,2)<<endl;
  affiche_liste(l3);
  cout<<endl;
  cout<<"Question 5"<<endl;
  cout<<"Liste l3 : ";
  affiche_liste(l3);
  insere_apres(l3,8,2);
  cout<<"Insertion 2 apres 8 dans l3 : ";
  affiche_liste(l3);
  cout<<"Question 5"<<endl;
  cout<<"Insertion 4 avant 2 dans l3 : ";
  insere_avant(l3,2,4);
  affiche_liste(l3);
  cout<<"Insertion 4 avant 7 dans l3 : ";
  insere_avant(l3,7,4);
  affiche_liste(l3);
  cout<<"Insertion 4 avant 1 dans l3 : ";
  insere_avant(l3,1,4);
  affiche_liste(l3);
  cout<<endl;
  cout<<"**************"<<endl;
  cout<<"* Exercice 3 *"<<endl;
  cout<<"**************"<<endl;
  cout<<endl;
  cout<<"Inverse de l3 : ";
  Liste l3inv=inverse(l3);
  affiche_liste(l3inv);
  cout<<"Inverse de l1 : ";
  Liste l1inv=inverse(l1);
  affiche_liste(l1inv);
}


